#!/usr/bin/env python3

import os
import json
import zipfile
import requests
import argparse

# 默认的 API 地址
DEFAULT_URL = "http://122.152.229.91:5005/api/hub/workflow/upload"

def read_metadata(workflow_dir):
    """
    从 metadata.json 中读取 workflow_id 和 version
    """
    metadata_path = os.path.join(workflow_dir, "metadata.json")
    
    if not os.path.exists(metadata_path):
        raise FileNotFoundError(f"在目录中未找到 metadata.json: {metadata_path}")
        
    try:
        with open(metadata_path, 'r', encoding='utf-8') as f:
            data = json.load(f)
            w_id = data.get('id')
            version = data.get('version')
            
            if not w_id or not version:
                raise ValueError("metadata.json 中缺少 'id' 或 'version' 字段")
                
            return w_id, version
    except json.JSONDecodeError:
        raise ValueError(f"metadata.json 格式错误，无法解析")

def zip_directory(source_dir, output_filename):
    """
    将整个目录打包成 zip 文件
    """
    print(f"📦 正在打包目录: {source_dir} -> {output_filename}")
    
    with zipfile.ZipFile(output_filename, 'w', zipfile.ZIP_DEFLATED) as zipf:
        for root, dirs, files in os.walk(source_dir):
            dirs[:] = [d for d in dirs if d != '__pycache__']
            for file in files:
                file_path = os.path.join(root, file)
                arcname = os.path.relpath(file_path, source_dir)
                
                if file == os.path.basename(output_filename):
                    continue
                    
                zipf.write(file_path, arcname)

def upload_workflow(workflow_id, version, zip_path, target_url):
    """
    上传 Zip 文件到指定接口
    """
    print(f"🚀 正在上传到: {target_url}")
    print(f"📄 Info: ID={workflow_id}, Version={version}, File={zip_path}")
    
    params = {
        "workflow_id": workflow_id,
        "version": version
    }
    
    try:
        with open(zip_path, 'rb') as f:
            files = {'file': (os.path.basename(zip_path), f, 'application/zip')}
            response = requests.post(target_url, params=params, files=files)
            
        if response.status_code == 200:
            print("✅ 上传成功！")
            print("服务器响应:", response.json())
        else:
            print(f"❌ 上传失败。状态码: {response.status_code}")
            print("错误信息:", response.text)
            
    except requests.exceptions.RequestException as e:
        print(f"❌ 网络请求错误: {e}")

def main():
    parser = argparse.ArgumentParser(description="自动打包并上传 Workflow")
    
    # 1. 位置参数：工作流目录 (必填)
    parser.add_argument("dir", help="工作流目录的路径")
    
    # 2. 可选参数：URL (选填，不填则使用 default)
    parser.add_argument(
        "--url", 
        default=DEFAULT_URL, 
        help=f"上传接口地址 (默认: {DEFAULT_URL})"
    )
    
    args = parser.parse_args()
    
    workflow_dir = args.dir
    target_url = args.url # 获取解析后的 URL

    temp_zip_name = ''
    
    try:
        # 读取元数据
        w_id, version = read_metadata(workflow_dir)

        temp_zip_name = f"{w_id}_{version}.zip"
        
        # 打包
        zip_directory(workflow_dir, temp_zip_name)
        
        # 上传
        upload_workflow(w_id, version, temp_zip_name, target_url)
        
    except Exception as e:
        print(f"❌ 发生错误: {e}")
    finally:
        if os.path.exists(temp_zip_name):
            os.remove(temp_zip_name)
            print("🧹 已清理临时文件")

if __name__ == "__main__":
    main()


from pathlib import Path
from typing import Literal, Optional
from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict

PROJECT_DEV_ROOT = Path(__file__).resolve().parent.parent.parent.parent

def auto_infer_dev_mode():
    if (PROJECT_DEV_ROOT / "pyproject.toml").exists():
        return "dev"
    return "prod"

class Settings(BaseSettings):
    model_config = SettingsConfigDict(
        env_file=".env",
        extra="ignore",
    )

    dev_mode: Literal["dev", "prod"] = Field(
        default_factory=auto_infer_dev_mode,
        description="Development mode. Set to 'dev' to enable development features.",
    )

    @property
    def is_dev(self) -> bool:
        return self.dev_mode == "dev"

    ## Log ##

    log_level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] = "INFO"

    ## Path settings ##

    # only available in dev mode
    project_dev_root: Path = PROJECT_DEV_ROOT

    root_dir: Optional[Path] = Path.home() / ".alaya.ai" / "alayaflow"

    workflows_dir: Optional[Path] = root_dir / "workflows"
    envs_dir: Optional[Path] = root_dir / "envs"
    logs_dir: Optional[Path] = root_dir / "logs"

    def set_root_dir(self, root_dir: Path | str):
        if isinstance(root_dir, str):
            root_dir = Path(root_dir)
        self.root_dir = root_dir
        self.workflows_dir = root_dir / "workflows"
        self.envs_dir = root_dir / "envs"
        self.logs_dir = root_dir / "logs"

    ## Service settings ##

    alayahub_url: str = "http://your-alayahub-url"
    alayahub_download_url: str = "http://your-alayahub-download-url"

    # Langfuse
    langfuse_enabled: bool = False
    langfuse_public_key: Optional[str] = ""
    langfuse_secret_key: Optional[str] = ""
    langfuse_base_url: Optional[str] = "http://100.64.0.34:4000/"

settings = Settings()


import json
import logging
from contextlib import asynccontextmanager
from typing import List, Dict, Optional, Any, Generic, TypeVar
from fastapi import FastAPI
from fastapi.responses import StreamingResponse
from pydantic import BaseModel, model_serializer

from alayaflow.api import Flow

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("alayaflow.server")

# =======================
# Pydantic 请求模型定义
# =======================

class InitConfig(BaseModel):
    config: Dict[str, Any] = {}

class LoadWorkflowRequest(BaseModel):
    workflow_id: str
    version: str
    init_args: Optional[Dict] = {}
    workflow_dir: Optional[str] = None

class InstallWorkflowRequest(BaseModel):
    workflow_id: str
    version: str

class ExecuteWorkflowRequest(BaseModel):
    workflow_id: str
    version: str
    inputs: Dict[str, Any]
    context: Dict[str, Any] = {}
    executor_type: str = "naive"
    metadata: Dict[str, Any] = {}

class ModelProfileRequest(BaseModel):
    profiles: List[Dict[str, Any]]

T = TypeVar("T")

class BaseResponse(BaseModel, Generic[T]):
    @model_serializer(mode='wrap')
    def serialize_model(self, handler) -> dict[str, Any]:
        dumped = handler(self)
        return {k: v for k, v in dumped.items() if v is not None}

    status: str
    data: Optional[T] = None
    error_msg: Optional[str] = None

    @classmethod
    def success(cls, data: Optional[T] = None) -> "BaseResponse[T]":
        return cls(status="success", data=data)

    @classmethod
    def error(cls, error_msg: str) -> "BaseResponse[None]":
        return cls(status="error", error_msg=error_msg)

# =======================
# FastAPI App Setup
# =======================

flow = Flow()

@asynccontextmanager
async def lifespan(app: FastAPI):
    logger.info("Initializing AlayaFlow API Singleton...")
    try:
        flow.init()     # all set in .env
        logger.info("Initialization successful.")
    except Exception as e:
        logger.error(f"Initialization failed: {e}")
        raise e
    
    yield
    
    logger.info("Shutting down AlayaFlow API Singleton...")

app = FastAPI(title="AlayaFlow Server", lifespan=lifespan)

# =======================
# API Endpoints
# =======================

@app.get("/health", response_model=BaseResponse[Dict])
async def health_check():
    return BaseResponse.success(data={"initialized": flow.is_inited()})


@app.get("/workflow/list_all", response_model=BaseResponse[Dict])
async def list_all_workflow():
    try:
        workflows = flow.list_all_workflows()
        return BaseResponse.success(data=workflows)
    except Exception as e:
        logger.error(f"Error listing all workflows: {e}", exc_info=True)
        return BaseResponse.error(error_msg=str(e))

@app.get("/workflows/list_loaded", response_model=BaseResponse[List[Dict]])
async def list_loaded_workflow():
    try:
        workflows = flow.list_loaded_workflows()
        return BaseResponse.success(data=workflows)
    except Exception as e:
        logger.error(f"Error listing loaded workflows: {e}", exc_info=True)
        return BaseResponse.error(error_msg=str(e))

@app.get("/workflow/list_local", response_model=BaseResponse[List[Dict]])
async def list_local_workflow():
    try:
        workflows = flow.list_local_workflows()
        return BaseResponse.success(data=workflows)
    except Exception as e:
        logger.error(f"Error listing local workflows: {e}", exc_info=True)
        return BaseResponse.error(error_msg=str(e))

@app.get("/workflow/list_remote", response_model=BaseResponse[List[Dict]])
async def list_remote_workflow():
    try:
        workflows = flow.list_remote_workflows()
        return BaseResponse.success(data=workflows)
    except Exception as e:
        logger.error(f"Error listing remote workflows: {e}", exc_info=True)
        return BaseResponse.error(error_msg=str(e))

@app.get("/workflow/info", response_model=BaseResponse[Dict])
async def get_workflow_info(workflow_id: str, version: str):
    try:
        info = flow.get_workflow_info(workflow_id, version)
        return BaseResponse.success(data=info)
    except Exception as e:
        logger.error(f"Error getting workflow info: {e}", exc_info=True)
        return BaseResponse.error(error_msg=str(e))

@app.post("/workflow/install", response_model=BaseResponse[Dict])
async def install_workflow(request: InstallWorkflowRequest):
    try:
        success = flow.install_workflow(request.workflow_id, request.version)
        if not success:
            raise BaseResponse.error(error_msg="Installation failed")
        return BaseResponse.success(data={
            "message": "Workflow installed successfully",
            "workflow_id": request.workflow_id
        })
    except Exception as e:
        logger.error(f"Error installing workflow: {e}")
        return BaseResponse.error(error_msg=str(e))

@app.post("/workflow/load", response_model=BaseResponse[Dict])
async def load_workflow(request: LoadWorkflowRequest):
    try:
        kwargs = {"init_args": request.init_args}
        if request.workflow_dir:
            kwargs["workflow_dir"] = request.workflow_dir
            
        flow.load_workflow(request.workflow_id, request.version, **kwargs)
        return BaseResponse.success(data={
            "message": "Workflow loaded successfully"
        })
    except Exception as e:
        logger.error(f"Error loading workflow: {e}")
        return BaseResponse.error(error_msg=str(e))

@app.post("/model/register", response_model=BaseResponse[Dict])
async def register_models(request: ModelProfileRequest):
    try:
        flow.register_models(request.profiles)
        return BaseResponse.success(data={"message": "Models registered successfully"})
    except Exception as e:
        logger.error(f"Error registering models: {e}")
        raise BaseResponse.error(error_msg=str(e))

@app.post("/workflow/execute/stream")
async def execute_workflow_stream(request: ExecuteWorkflowRequest):
    """
    流式执行工作流 (NDJSON 风格)
    """
    async def event_generator():
        try:
            async for event in flow.exec_workflow_async(
                workflow_id=request.workflow_id,
                version=request.version,
                inputs=request.inputs,
                context=request.context,
                executor_type=request.executor_type,
                metadata=request.metadata
            ):
                yield json.dumps(event, ensure_ascii=False) + "\n"
        except Exception as e:
            logger.error(f"Error during streaming execution: {e}")
            error_event = {"event": "error", "error": str(e)}
            yield json.dumps(error_event, ensure_ascii=False) + "\n"

    return StreamingResponse(event_generator(), media_type="application/x-ndjson")

@app.post("/workflows/execute/sync", response_model=BaseResponse[Dict])
async def execute_workflow_sync(request: ExecuteWorkflowRequest):
    results = []
    try:
        async for event in flow.exec_workflow_async(
            workflow_id=request.workflow_id,
            version=request.version,
            inputs=request.inputs,
            context=request.context,
            executor_type=request.executor_type,
            metadata=request.metadata
        ):
            results.append(event)
        return BaseResponse.success(data=results)
    except Exception as e:
        logger.error(f"Error during sync execution: {e}", exc_info=True)
        raise BaseResponse.error(error_msg=str(e))

if __name__ == "__main__":
    import uvicorn
    uvicorn.run(app, host="0.0.0.0", port=8000)


import json
import zipfile
import shutil
import tempfile

from pathlib import Path
from typing import Dict, List, Optional, Any
import requests

from alayaflow.utils.logger import AlayaFlowLogger
from alayaflow.workflow.workflow_info import WorkflowKey, WorkflowInfo
from alayaflow.workflow.workflow_loader import WorkflowLoader
from alayaflow.workflow.runnable import BaseRunnableWorkflow
from alayaflow.clients.alayahub.http_client import HttpAlayaHubClient
from alayaflow.common.config import settings


logger = AlayaFlowLogger()


class WorkflowManager:
    def __init__(self):
        self._info_cache: Dict[WorkflowKey, WorkflowInfo] = {}
        self._workflow_cache: Dict[WorkflowKey, BaseRunnableWorkflow] = {}
        self._requirements_cache: Dict[WorkflowKey, List[str]] = {}
        
    def clear_cache(self):
        self._info_cache.clear()
        self._workflow_cache.clear()
        self._requirements_cache.clear()
    
    def load_workflow(self, workflow_id: str, version: str, init_args: Optional[Dict] = None, workflow_dir: Optional[str] = None) -> None:
        if init_args is None:
            init_args = {}
        if workflow_dir is None:
            workflow_dir = settings.workflows_dir / workflow_id / version
        else:
            workflow_dir = Path(workflow_dir)
        loader = WorkflowLoader(workflow_dir, workflow_id, version)
        runnable = loader.load_workflow(init_args)
        requirements = loader.load_requirements()
        self._register_workflow(runnable, requirements)

    def unload_workflow(self, workflow_id: str, version: str) -> None:
        key = WorkflowKey(workflow_id, version)
        if key in self._workflow_cache:
            del self._workflow_cache[key]
        if key in self._info_cache:
            del self._info_cache[key]
        if key in self._requirements_cache:
            del self._requirements_cache[key]

    def register_workflow(self, runnable: BaseRunnableWorkflow, requirements: List[str] = None) -> None:
        self._register_workflow(runnable, requirements or [])

    def _register_workflow(self, runnable: BaseRunnableWorkflow, requirements: List[str]) -> None:
        info = runnable.info
        key = WorkflowKey(info.id, info.version)
        self._info_cache[key] = info
        self._workflow_cache[key] = runnable
        self._requirements_cache[key] = requirements

    # Keep for future reference
    # def load_workflows(self) -> None:
    #     if not settings.workflow_storage_path.exists():
    #         raise FileNotFoundError(f"Workflow storage path {settings.workflow_storage_path} does not exist")
    #     for workflow_id_dir in settings.workflow_storage_path.iterdir():
    #         if not workflow_id_dir.is_dir() or workflow_id_dir.name.startswith("_"):
    #             continue
    #         workflow_id = workflow_id_dir.name
    #         for version_dir in workflow_id_dir.iterdir():
    #             if not version_dir.is_dir() or version_dir.name.startswith("_"):
    #                 continue
                
    #             version = version_dir.name
    #             try:
    #                 self.load_workflow(workflow_id, version)
    #             except (FileNotFoundError, ValueError, IOError, ImportError, AttributeError, TypeError) as e:
    #                 print(f"Loading workflow {workflow_id} version {version} failed: {e}")
    #                 continue

    def get_workflow_info(self, workflow_id: str, version: str) -> WorkflowInfo:
        key = WorkflowKey(workflow_id, version)
        info = self._info_cache.get(key)
        if info is None:
            workflow_dir = settings.workflows_dir / workflow_id / version
            loader = WorkflowLoader(workflow_dir, workflow_id, version)
            info = loader.load_info()
            self._info_cache[key] = info
        return info

    def get_workflow_requirements(self, workflow_id: str, version: str) -> List[str]:
        key = WorkflowKey(workflow_id, version)
        reqs = self._requirements_cache.get(key)
        if reqs is None:
            workflow_dir = settings.workflows_dir / workflow_id / version
            loader = WorkflowLoader(workflow_dir, workflow_id, version)
            reqs = loader.load_requirements()
            self._requirements_cache[key] = reqs
        return reqs

    def get_workflow_runnable(self, workflow_id: str, version: str) -> BaseRunnableWorkflow:
        key = WorkflowKey(workflow_id, version)
        runnable = self._workflow_cache.get(key)
        if runnable is None:
            raise ValueError(f"Workflow {workflow_id} version {version} does not loaded. Call load_workflow first.")
        return runnable

    ## Loaded Workflows ##

    def list_loaded_workflows(self) -> List[str]:
        return sorted([f'{wf.id}:{wf.version}' for wf in self._workflow_cache.keys()])

    def list_loaded_workflow_ids(self) -> List[str]:
        return sorted([wf.id for wf in self._workflow_cache.keys()])

    def list_loaded_workflow_versions(self, workflow_id: str) -> List[str]:
        return sorted([wf.version for wf in self.list_loaded_workflow_metas() if wf.id == workflow_id])

    def list_loaded_workflow_metas(self) -> List[WorkflowInfo]:
        return sorted([wf.info for wf in self._workflow_cache.values()])

    ## Local Workflows ##

    def list_local_workflow_metas(self) -> List[WorkflowInfo]:
        if not settings.workflows_dir.exists():
            raise FileNotFoundError(f"Workflow storage path {settings.workflows_dir} does not exist")
        metas = []
        for workflow_id_dir in settings.workflows_dir.iterdir():
            if not workflow_id_dir.is_dir() or workflow_id_dir.name.startswith("_"):
                continue
            workflow_id = workflow_id_dir.name
            for version_dir in workflow_id_dir.iterdir():
                if not version_dir.is_dir() or version_dir.name.startswith("_"):
                    continue
                
                version = version_dir.name
                try:
                    meta = self.get_workflow_info(workflow_id, version)
                    metas.append(meta)
                except (FileNotFoundError, ValueError, IOError, ImportError, AttributeError, TypeError) as e:
                    print(f"Loading workflow {workflow_id} version {version} failed: {e}")
                    continue
        sorted_metas = sorted(metas, key=lambda x: (x.id, x.version))
        return sorted_metas

    def list_local_workflows(self) -> List[str]:
        return sorted([f'{wf.id}:{wf.version}' for wf in self.list_local_workflow_metas()])
        
    def list_local_workflow_ids(self) -> List[str]:
        return sorted([wf.id for wf in self.list_local_workflow_metas()])

    def list_local_workflow_versions(self, workflow_id: str) -> List[str]:
        return sorted([wf.version for wf in self.list_local_workflow_metas() if wf.id == workflow_id])

    ## Remote Workflows ##

    def list_remote_workflow_metas(self) -> List[WorkflowInfo]:
        metas = []
        try:
            list_url = f"{settings.alayahub_url}/api/hub/workflow/list"
            logger.info(f"Requesting remote workflows from {list_url}")
            response = requests.get(list_url, timeout=10)
            response.raise_for_status()
            resp = response.json()
            if not isinstance(resp, list):
                logger.error(f"Unexpected response format: {resp}")
                return []
            metas = [WorkflowInfo.from_dict(item) for item in resp]
        except requests.RequestException as e:
            logger.error(f"Failed to list remote workflows: {e}", exc_info=True)
            return []

        return sorted(metas, key=lambda x: (x.id, x.version))

    def list_remote_workflows(self) -> List[str]:
        return sorted([f'{wf.id}:{wf.version}' for wf in self.list_remote_workflow_metas()])

    def list_remote_workflow_ids(self) -> List[str]:
        return sorted([wf.id for wf in self.list_remote_workflow_metas()])

    def list_remote_workflow_versions(self, workflow_id: str) -> List[str]:
        return sorted([wf.version for wf in self.list_remote_workflow_metas() if wf.id == workflow_id])

    def list_all_workflows(self) -> Dict[str, List[str]]:
        """列举工作流，分别返回已加载、本地已安装和远程未安装的工作流"""
        loaded_workflows = self.list_loaded_workflows()
        local_workflows = self.list_local_workflows()
        remote_workflows = self.list_remote_workflows()
        return {
            "loaded": loaded_workflows,
            "local": local_workflows,
            "remote": remote_workflows,
        }

    def install_workflow(self, workflow_id: str, version: str, force_reinstall: bool = False, workflow_dir: Optional[str] = None) -> bool:
        try:
            if not force_reinstall and f"{workflow_id}:{version}" in self.list_local_workflows():
                logger.info(f"Workflow {workflow_id}:{version} is already loaded. Use force_reinstall=True to reinstall.")
                return False
            
            self.uninstall_workflow(workflow_id, version, raise_error=False, workflow_dir=workflow_dir)
            
            # 创建临时目录存放 zip 文件
            with tempfile.TemporaryDirectory() as temp_dir:
                zip_path = Path(temp_dir) / f"{workflow_id}.zip"

                client = HttpAlayaHubClient(settings.alayahub_url, settings.alayahub_download_url)
                client.download_workflow(workflow_id, version, zip_path)
                
                # 读取 metadata.json 获取工作流信息
                with zipfile.ZipFile(zip_path, "r") as zip_ref:
                    # 检查是否有 metadata.json
                    metadata_content = None
                    for file_info in zip_ref.namelist():
                        if file_info.endswith("metadata.json"):
                            metadata_content = zip_ref.read(file_info).decode("utf-8")
                            break
                    
                    if not metadata_content:
                        raise ValueError(f"工作流 {workflow_id} 的 zip 文件中未找到 metadata.json")
                    
                    # 解析 metadata.json 获取版本
                    metadata = json.loads(metadata_content)
                    version = metadata.get("version", "1.0.0")
                    
                    # 确定本地工作流目录
                    if workflow_dir:
                        workflow_dir: Path = Path(workflow_dir)
                    else:
                        workflow_dir: Path = settings.workflows_dir / workflow_id / version
                    
                    # 5. 创建目录并解压文件
                    workflow_dir.mkdir(parents=True, exist_ok=True)
                    
                    # 解压所有文件到目标目录
                    zip_ref.extractall(workflow_dir)

                    logger.info(f"工作流 {workflow_id}:{version} 已成功安装到 {workflow_dir.absolute()}")
                    
                    return True
                    
        except requests.RequestException as e:
            raise ValueError(f"下载工作流 {workflow_id} 失败: {e}") from e
        except Exception as e:
            raise ValueError(f"安装工作流 {workflow_id} 失败: {e}") from e
    
    def uninstall_workflow(self, workflow_id: str, version: str = None, raise_error: bool = True, workflow_dir: Optional[str] = None) -> bool:
        try:
            self.unload_workflow(workflow_id, version)

            if workflow_dir:
                workflow_dir = Path(workflow_dir)
            else:
                if version:
                    workflow_dir = settings.workflows_dir / workflow_id / version
                else:
                    workflow_dir = settings.workflows_dir / workflow_id  # 未指定版本时，删除整个工作流目录
            
            if not workflow_dir.exists() or not workflow_dir.is_dir():
                if raise_error:
                    if version:
                        raise ValueError(f"工作流 {workflow_id}:{version} 未安装")
                    else:
                        raise ValueError(f"工作流 {workflow_id} 未安装")
                else:
                    return False
            
            shutil.rmtree(workflow_dir)
            
            return True
            
        except Exception as e:
            raise ValueError(f"卸载工作流 {workflow_id} 失败: {e}") from e



from typing import Generator, Dict, List, Self, Optional, AsyncGenerator

from alayaflow.utils.singleton import SingletonMeta
from alayaflow.workflow import WorkflowManager
from alayaflow.execution import ExecutorManager, ExecutorType
from alayaflow.common.config import settings
from alayaflow.component.model import ModelManager, ModelProfile
from alayaflow.workflow.runnable import BaseRunnableWorkflow
from alayaflow.utils.logger import AlayaFlowLogger


logger = AlayaFlowLogger()


class APISingleton(metaclass=SingletonMeta):
    def __init__(self) -> None:
        self.workflow_manager = WorkflowManager()
        self.executor_manager = ExecutorManager(
            workflow_manager=self.workflow_manager
        )
        self.model_manager = ModelManager()
        self._inited = False                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  

    def is_inited(self) -> bool:
        return self._inited

    def _check_init(self) -> None:
        if not self._inited:
            raise ValueError("Flow APISingleton is not initialized. Please call init() first.")

    def init(self, config: dict = None) -> Self:
        """初始化 Flow APISingleton"""
        if self._inited:
            logger.warning("Flow APISingleton is already initialized. Will override the existing config.")

        if config:
            for key, value in config.items():
                if hasattr(settings, key):
                    setattr(settings, key, value)       # will be check by pydantic validator
                else:
                    logger.warning(f"Config key '{key}' is not a valid setting field. Ignored.")

        logger.info(f"AlayaFlow is initialized with config: {settings.model_dump()}")
        
        self._inited = True
        return self

    def list_registered_models(self) -> List[ModelProfile]:
        """列举已注册的模型"""
        self._check_init()
        return self.model_manager.list_registered_models()

    def register_models(self, model_profiles: List[dict | ModelProfile], overwrite: bool = False) -> None:
        self._check_init()
        profiles = [ModelProfile(**profile) if isinstance(profile, dict) else profile for profile in model_profiles]
        for profile in profiles:
            self.model_manager.register_profile(profile, overwrite=overwrite)

    def list_loaded_workflows(self) -> List[str]:
        """列举已加载的工作流"""
        self._check_init()
        return self.workflow_manager.list_loaded_workflows()

    def list_local_workflows(self) -> List[str]:
        """列举本地已安装的工作流"""
        self._check_init()
        return self.workflow_manager.list_local_workflows()

    def list_remote_workflows(self) -> List[str]:
        """列举远程未安装的工作流"""
        self._check_init()
        return self.workflow_manager.list_remote_workflows()

    def list_all_workflows(self) -> Dict[str, List[str]]:
        """列举工作流，分别返回本地已安装和远程未安装的工作流"""
        self._check_init()
        return self.workflow_manager.list_all_workflows()

    def get_workflow_info(self, workflow_id: str, version: str) -> dict:
        """获取工作流信息"""
        self._check_init()
        return self.workflow_manager.get_workflow_info(workflow_id, version).to_dict()

    def install_workflow(self, workflow_id: str, version: str, force_reinstall: bool = False) -> bool:
        """安装工作流"""
        self._check_init()
        return self.workflow_manager.install_workflow(workflow_id, version, force_reinstall)

    def uninstall_workflow(self, workflow_id: str) -> bool:
        """卸载工作流"""
        self._check_init()
        return self.workflow_manager.uninstall_workflow(workflow_id)

    def load_workflow(self, workflow_id: str, version: str, init_args: Optional[Dict] = None, workflow_dir: Optional[str] = None) -> None:
        """加载本地工作流"""
        self._check_init()
        return self.workflow_manager.load_workflow(workflow_id, version, init_args or {}, workflow_dir)

    def register_workflow(self, runnable: BaseRunnableWorkflow, requirements: List[str] = None) -> None:
        """加载Python项目中的工作流"""
        self._check_init()
        self.workflow_manager.register_workflow(runnable, requirements or [])

    def install_then_load_workflow(self, workflow_id: str, version: str, init_args: Optional[Dict] = None, force_reinstall: bool = False, workflow_dir: Optional[str] = None) -> None:
        """安装并加载工作流"""
        self._check_init()
        self.workflow_manager.install_workflow(workflow_id, version, force_reinstall, workflow_dir)
        self.workflow_manager.load_workflow(workflow_id, version, init_args or {}, workflow_dir)

    def exec_workflow(
        self,
        workflow_id: str,
        version: str,
        inputs: dict,
        context: dict,
        executor_type: str | ExecutorType = ExecutorType.NAIVE,
        metadata: dict = None
    ) -> Generator[dict, None, None]:
        """执行工作流"""
        self._check_init()
        for event in self.executor_manager.exec_workflow(
            workflow_id=workflow_id,
            version=version,
            inputs=inputs,
            context=context,
            executor_type=executor_type,
            metadata=metadata
        ):
            yield event

    async def exec_workflow_async(
        self,
        workflow_id: str,
        version: str,
        inputs: dict,
        context: dict,
        executor_type: str | ExecutorType = ExecutorType.NAIVE,
        metadata: dict = None
    ) -> AsyncGenerator[dict, None]:
        """异步执行工作流"""
        self._check_init()
        async for event in self.executor_manager.exec_workflow_async(
            workflow_id=workflow_id,
            version=version,
            inputs=inputs,
            context=context,
            executor_type=executor_type,
            metadata=metadata
        ):
            yield event



import requests

from alayaflow.utils.logger import AlayaFlowLogger

logger = AlayaFlowLogger()

class HttpAlayaHubClient:
    def __init__(self, base_url: str, download_url: str):
        """Initialize the HTTP client with base URL"""
        self.base_url = base_url.rstrip('/')
        self.download_url = download_url.rstrip('/')

    def download_workflow(self, workflow_id: str, version: str, output_path: str):
        """Download a workflow from the hub"""
        download_url = f"{self.download_url}/workflows/{workflow_id}/{version}/download/workflows.zip"

        logger.info(f"Downloading workflow {workflow_id} version {version} from {download_url} to {output_path}")

        response = requests.get(download_url, stream=True, timeout=30)
        response.raise_for_status()
        
        with open(output_path, 'wb') as f:
            for chunk in response.iter_content(chunk_size=8192):
                f.write(chunk)
